﻿#extension GL_OES_EGL_image_external : require

precision mediump float;
varying vec2 vUV;
uniform vec4 uColor;
uniform sampler2D  sEffectTexture;
uniform sampler2D  sNextEffectTexture;
uniform samplerExternalOES sTexture;
uniform float uSwipePosX;

const float width = 8.0;
const float widthM1 = width - 1.0;
const float sliceSize     = 1.0 / width;              // space of 1 slice',
const float slicePixelSize    = sliceSize / width;            // space of 1 pixel',
const float sliceInnerSize    = slicePixelSize * (width - 1.0);   // space of width pixels'c,

vec4 sampleAs3DTexture(sampler2D texture, vec3 uv)
{
    float zSlice0   = min(floor(uv.z * width), widthM1);
    float zSlice1   = min(zSlice0 + 1.0, widthM1);
    float xOffset   = slicePixelSize * 0.5 + uv.x * sliceInnerSize;
    float s0    = xOffset + (zSlice0 * sliceSize);
    float s1    = xOffset + (zSlice1 * sliceSize);
    vec4 slice0Color= texture2D(texture, vec2(s0, uv.y));
    vec4 slice1Color= texture2D(texture, vec2(s1, uv.y));
    float zOffset   = mod(uv.z * width, 1.0);
    vec4 result = mix(slice0Color, slice1Color, zOffset);
    return result;
}

vec2 KaleidoscopeIt(vec2 uv)
{
    float sides = 8.0;
    float angle = 0.0;

    vec2 p = uv - 0.5;
    float r = length(p);
    float a = atan(p.y, p.x) + angle;
    float tau = 2. * 3.1416;
    a = mod(a, tau/sides);
    a = abs(a - tau/sides/2.);
    p = r * vec2(cos(a), sin(a));

    return p + 0.5;
}

void main()
{
    // read the screen texture color
    vec2 newUV = KaleidoscopeIt(vUV);
    newUV = vUV;
    vec4 srcColor = texture2D(sTexture, newUV);

    // read matching color in tColorCube0 and tColorCube1
    vec4 color;
    if (gl_FragCoord.x > uSwipePosX)
    {
//      color = srcColor;
        color = sampleAs3DTexture(sNextEffectTexture, srcColor.rgb );
    }
    else
    {
        color = sampleAs3DTexture(sEffectTexture, srcColor.rgb );
    }

    // mix colors from each color cubes, and keep the alpha from original screen
   gl_FragColor = vec4(color.rgb, srcColor.a) * uColor;
}

