﻿#extension GL_OES_EGL_image_external : require

precision mediump float;
varying vec2 vUV;
uniform vec4 uColor;
uniform sampler2D  sEffectTexture;
uniform samplerExternalOES sTexture;


vec4 sampleAs3DTexture(sampler2D texture, vec3 uv, float width)
{
    float sliceSize     = 1.0 / width;              // space of 1 slice',
    float slicePixelSize    = sliceSize / width;            // space of 1 pixel',
    float sliceInnerSize    = slicePixelSize * (width - 1.0);   // space of width pixels',
    float zSlice0   = min(floor(uv.z * width), width - 1.0);
    float zSlice1   = min(zSlice0 + 1.0, width - 1.0);
    float xOffset   = slicePixelSize * 0.5 + uv.x * sliceInnerSize;
    float s0    = xOffset + (zSlice0 * sliceSize);
    float s1    = xOffset + (zSlice1 * sliceSize);
    vec4 slice0Color= texture2D(texture, vec2(s0, uv.y));
    vec4 slice1Color= texture2D(texture, vec2(s1, uv.y));
    float zOffset   = mod(uv.z * width, 1.0);
    vec4 result = mix(slice0Color, slice1Color, zOffset);
    return result;
}

void main()
{
    // read the screen texture color
    vec4 srcColor = texture2D(sTexture, vUV);

    // read matching color in tColorCube0 and tColorCube1
    vec4 color = sampleAs3DTexture(sEffectTexture, srcColor.rgb, 8.0);

    // mix colors from each color cubes, and keep the alpha from original screen
   gl_FragColor = vec4(color.rgb, srcColor.a) * uColor;
}