﻿precision mediump float;
varying vec2 vUV;
uniform vec4 uColor;
uniform sampler2D sTexture;
uniform sampler2D  sEffectTexture;
uniform sampler2D  sNextEffectTexture;
uniform float  uSwipePosX;

const float width = 8.0;
const float widthM1 = width - 1.0;
const float sliceSize     = 1.0 / width;              // space of 1 slice',
const float slicePixelSize    = sliceSize / width;            // space of 1 pixel',
const float sliceInnerSize    = slicePixelSize * (width - 1.0);   // space of width pixels'c,

vec4 sampleAs3DTexture(sampler2D texture, vec3 uv)
{
    float zSlice0   = min(floor(uv.z * width), widthM1);
    float zSlice1   = min(zSlice0 + 1.0, widthM1);
    float xOffset   = slicePixelSize * 0.5 + uv.x * sliceInnerSize;
    float s0    = xOffset + (zSlice0 * sliceSize);
    float s1    = xOffset + (zSlice1 * sliceSize);
    vec4 slice0Color= texture2D(texture, vec2(s0, uv.y));
    vec4 slice1Color= texture2D(texture, vec2(s1, uv.y));
    float zOffset   = mod(uv.z * width, 1.0);
    vec4 result = mix(slice0Color, slice1Color, zOffset);
    return result;
}

void main()
{
	vec4 color1 = texture2D(sTexture, vUV) * uColor;
    vec4 color2;
    if (gl_FragCoord.x > uSwipePosX)
    {
        color2 = sampleAs3DTexture(sNextEffectTexture, color1.rgb );
    }
    else
    {
        color2 = sampleAs3DTexture(sEffectTexture, color1.rgb );
    }

    gl_FragColor = vec4(color2.rgb, uColor.a * color1.a);
}
